<?php declare(strict_types = 1);

namespace Drupal\activity_stream;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\activity_stream\ActivityDestinationInterface;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\Core\Entity\ContentEntityTypeInterface;
use Drupal\field\FieldConfigInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * @todo Add class description.
 */
final class Helper {

  use LoggerChannelTrait;
  use StringTranslationTrait;

  /**
   * Constructs a Helper object.
   */
  public function __construct(
    private readonly EntityTypeManagerInterface $entityTypeManager,
    private readonly EntityFieldManagerInterface $entityFieldManager,
    private readonly ConfigFactoryInterface $configFactory,
  ) {}

  /**
   * Undocumented function
   *
   * @param [type] $entity
   * @param [type] $recipients
   * @param [type] $destinations
   * @param [type] $activity_date
   * @return void
   */
  public function createActivity($entity, $recipients, $destinations, $activity_date = NULL): void {
    // Read the entity Type from the Entity
    $entity_type_id = $entity->getEntityTypeId();
    // Read current user
    $current_user = \Drupal::currentUser();

    $storage_activity = $this->entityTypeManager->getStorage('activity_stream_activity')->create();

    $storage_activity->field_activity_entity->target_type = $entity_type_id;
    $storage_activity->field_activity_entity->target_id = $entity->id();

    $users = $this->entityTypeManager->getStorage('user')->loadMultiple($recipients);

    if ($activity_date !== NULL) {
      $storage_activity->set('field_activity_date', $activity_date);
    }
    else {
      $storage_activity->set('field_activity_date', time());
    }

    if (isset($users) && is_array($users) && !empty($users)) {
      foreach($users as $user) {
        $storage_activity->field_activity_recipient_user[] = $user->id(); 
      }
    }

    if(isset($destinations) && is_array($destinations) && !empty($destinations)) {
      foreach($destinations as $key => $destination) {
        if ($this->validateDestination($destination)) {
          $storage_activity->field_activity_destinations[] = $destination; 
        }
      }
    }

    $violations = $storage_activity->validate();
    if ($violations->count() > 0) {
      $this->getLogger('activity_stream')->error('Activity was not created due to validation errors. This usally happens when you miss out some required fields.');
    }
    else {
      $storage_activity->save();
    }     

  }

  /**
   * Validate destination
   *
   * @param [type] $destination
   * @return boolean
   */
  protected function validateDestination($destination): bool {

    $activity_destination = $this->entityTypeManager->getStorage('activity_destination')->load($destination);

    if ($activity_destination instanceof ActivityDestinationInterface) {
      return TRUE;
    }

    return FALSE;

  }

  public function getSupportedContentEntities() {
    $entity_type_manager = $this->entityTypeManager;
    $options = [];
    foreach ($entity_type_manager->getDefinitions() as $entity_id => $entity_type) {
      // Content entity.
      if ($entity_type instanceof ContentEntityTypeInterface
        && $entity_id !== 'activity'
        && $entity_id !== 'message'
        && $entity_id !== 'message_template') {
        $entity_type_bundle_info = \Drupal::service('entity_type.bundle.info');
        $config_entity_bundles = $entity_type_bundle_info->getBundleInfo($entity_type->id());
        foreach ($config_entity_bundles as $key => $value) {
          // Dot character in key names is not allowed in config, so we use "-".
          $options[$entity_id . '-' . $key] = $entity_type->getLabel() . ': ' . $value['label'];
        }
      }
    }
    asort($options);
    return $options;
  }

  public function getSupportedMessageTemplates() {

    $message_templates = [
      '_none' => t('Do not use a message template')
    ];

    // Message type storage.
    $message_storage = $this->entityTypeManager->getStorage('message_template');

    // Check all enabled messages.
    foreach ($message_storage->loadByProperties(['status' => '1']) as $key => $message_type) {
      if ($this->checkActivityStreamSupport($key)) {
        $message_templates[$key] = $key;
      }
    }

    return $message_templates;


  }

  public function getSupportedDateFields($bundles) {

    $supported_fields = ['_none' => $this->t('Do not use Entity date field for activity date')];

    if (isset($bundles) && is_array($bundles) && !empty($bundles)) {
      //$count = count($bundles);
      //if ($count === 1) {

        foreach ($bundles as $key => $bundle) {
          $bundle_string = $key; 
        }

        // Get entity type id
        $entity_type_id = $this->getEntityTypeIdFromString($bundle_string);
        $entity_bundle  = $this->getEntityBundleFromString($bundle_string);
        // Check on that bundle
        $fields = $this->entityFieldManager
          ->getFieldDefinitions($entity_type_id, 
                              $entity_bundle);

        foreach ($fields as $key => $field) {
          if ($field->getType() === 'datetime' || $field->getType() === 'smartdate') {  
            $supported_fields[$key] = $field->getName();
          }
        }

      //}
    }

    return $supported_fields;

  }

  protected function getEntityTypeIdFromString($string) {
    $pos = strpos($string, '-');
    $entity_type_id = substr($string, 0, $pos);
    return $entity_type_id;
  }

  protected function getEntityBundleFromString($string) {
    $pos = strpos($string, '-') + 1;
    $entity_bundle = substr($string, $pos);
    return $entity_bundle;
  }  

  protected function checkActivityStreamSupport(string $message_type): bool {
    $config_storage = $this->entityTypeManager
      ->getStorage('field_config');

    $field_message_context = $config_storage->load("message.$message_type.field_message_context");
    if (!$field_message_context instanceof FieldConfigInterface) {
      return FALSE;
    }

    $field_message_destination = $config_storage->load("message.$message_type.field_message_destination");
    if (!$field_message_destination instanceof FieldConfigInterface) {
      return FALSE;
    }
    
    $field_message_related_object = $config_storage->load("message.$message_type.field_message_related_object");
    if (!$field_message_related_object instanceof FieldConfigInterface) {
      return FALSE;
    } 
    
    return TRUE;

  }


}
